//
// Copyright (C) 2013-2016  Taiyou Sakashita, Hiroko Yamada
//
// Calculate Monthlymean 2015/08/10
// Modified for using Julidan day 2016/07/01
//

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <time.h>
#include <math.h>
#include "bin.c"


// Length of the Data in Column/Line Direction
int X;
int Y;

// Year/Month/Day
int YYYY;
int MM;
int DD;

// Nubmer of Used Files
int nFile = 0;

// Number of Used Files for Standard Deviation
int nFile_std = 0;

// Error Day Threshold
int Error_DD_THS = 7;

// Present Error Day
int P_Error_DD;

// Monthlymean
double *MOM;

// Standard Deviation
double *STD;

// File Type
char ftype[512];

// Directory Name to Read Dailymean Data
char DayMn_DIR[512];

// Directory Name to Write Monthlymean Data
char MonthlyMn_DIR[512];

/// TIME Code 
char TIME_Code[512];

void cal_monthlymean(char *DIR);
void cal_std(char *DIR);
void add_data(double *RD);
void cal_mean_d(double *DD, int N, double *RR);
void cal_var(float *DR);
void standard_deviation();
int get_lastday(int Year, int Month);
void date_errorcheck();
void initialization();
void read_formatfile();


int main(int argc, char *argv[])
{


  int I;

  // Filename to Write Monthlymean Data
  char wfilename[512];

  // Filename to Write Standard Deviation Data
  char std_filename[512];

  // Year/Month
  YYYY = atoi(argv[1]);
  MM   = atoi(argv[2]);
//check & make output/MONTH directory
  struct stat  st;
  char fname[20];
  char str1[20] = "output/";
  sprintf(fname,"%s%d%02d",str1,YYYY,MM);
        
//check & make output/MONTH directory
        if (stat(fname, &st) != 0){
            printf("%s dosen't exist\n", fname);
            if(mkdir(fname, 0777)==0){
                printf("Succeeded to create a directory\n");
            }else{
                printf("Failed to create a directory\n");
            }
        }
        else {
                switch (st.st_mode & S_IFMT) {
                        case S_IFREG:
                                printf("%s is a file\n", fname);
                                break;
                        case S_IFDIR:
                                printf("%s is a dir\n", fname);
                                break;
                        default:
                                printf("%s isn't a file or dir\n", fname);
                        break;
                }
        }
        
  date_errorcheck();
  initialization();


  // Calculate Monthlymean
  cal_monthlymean(DayMn_DIR);

  char ftype_cut[512]="";
  strncpy(ftype_cut,ftype,strlen(ftype)-4);
  *(ftype_cut+strlen(ftype)-4) = 0;

//  sprintf(wfilename, "%s%d%02d/%d%02d.%s%s.mm.bin", MonthlyMn_DIR, YYYY, MM, YYYY, MM, TIME_Code, ftype);
  sprintf(wfilename, "%s%d%02d/%d%02d.%s%s.mm.bin", MonthlyMn_DIR, YYYY, MM, YYYY, MM, TIME_Code, ftype_cut);


  // Write Monthlymean Data
  if(nFile > 0)
  {
     d2bin(wfilename, X, Y, MOM);

     // Calculate Standard Deviation
     cal_std(DayMn_DIR);

     // Filename cut 
     char ftype_cut[512] = "";
     strncpy(ftype_cut, ftype,strlen(ftype)-4);
     *(ftype_cut+strlen(ftype)-4) = 0;         /* string end null */

//     sprintf(std_filename, "%s%d%02d/%d%02d.%s%s.std.bin", MonthlyMn_DIR, YYYY, MM, YYYY, MM, TIME_Code, ftype);
     sprintf(std_filename, "%s%d%02d/%d%02d.%s%s.std.bin", MonthlyMn_DIR, YYYY, MM, YYYY, MM, TIME_Code, ftype_cut);

     // Write Standard Deviation Data
     if(nFile_std > 0) d2bin(std_filename, X, Y, STD);

  }//fi


  free(MOM);
  free(STD);


  return 0;


}//main


void cal_monthlymean(char *DayMn_DIR)
{

  // Last Day of the Month
  int Last_DD;

  // Dailymean
  double *DM;

  // Filename to Read Dailymean Data
  char filename[512];

  struct stat st;


  if((DM = (double *)malloc(sizeof(double)*((X+1)*(Y+1)))) == NULL) {printf("malloc failed."); exit(1);}

  P_Error_DD = 0;

  Last_DD = get_lastday(YYYY, MM);

  // Filename cut 
  char ftype_cut[512];
  strncpy(ftype_cut, ftype, strlen(ftype)-4);
  *(ftype_cut+strlen(ftype)-4) = 0;  /* string end null */
  
 // Calculate Sum of Dailymean Data
  for(DD=1;DD<=Last_DD;DD++)
  {
     sprintf(filename, "%s%d%02d/%d%02d%02d.%s%s.dm.bin", DayMn_DIR, YYYY, MM, YYYY, MM, DD, TIME_Code, ftype_cut);

     if(stat(filename, &st) == 0)
     {
        bin2d(filename, X, Y, DM);
        add_data(DM);
        nFile++;

     }else{

        P_Error_DD++;
        printf("%s is not found\n", filename);

     }//fi

  }//DD


  if(P_Error_DD >= Error_DD_THS)
  {
     printf("Loss of dailymean is too much on %d%02d\n", YYYY, MM);
     exit(1);

  }//fi


  // Calculate Monthlymean
  if(nFile > 0) cal_mean_d(MOM, nFile, MOM);


  free(DM);


  return;


}//cal_monthlymean


void cal_std(char *DayMn_DIR)
{

  // Last Day of the Month
  int Last_DD;

  // Data Read
  float *DR;

  // Filename of Original Data
  char filename[512];

  struct stat st;


  if((DR = (float *)malloc(sizeof(float)*((X+1)*(Y+1)))) == NULL) {printf("malloc failed."); exit(1);}


  Last_DD = get_lastday(YYYY, MM);

  // Filename cut 
  char ftype_cut[512];
  strncpy(ftype_cut, ftype, strlen(ftype)-4);
  *(ftype_cut + strlen(ftype)-4) = 0;   /* string end null */

  // Calculate Variance
  for(DD=1;DD<=Last_DD;DD++)
  {
     sprintf(filename, "%s%d%02d/%d%02d%02d.%s%s.dm.bin", DayMn_DIR, YYYY, MM, YYYY, MM, DD, TIME_Code, ftype_cut);

     if(stat(filename, &st) == 0)
     {
        bin2f(filename, X, Y, DR);
        cal_var(DR);
        nFile_std++;

     }//fi
  }//DD

  // Calculate Standard Deviation
  if(nFile_std > 0)
  {
     cal_mean_d(STD, nFile_std, STD);
     standard_deviation();

  }//fi


  free(DR);


  return;


}//cal_std


void add_data(double *DR)
{

  int I,J;

  for(I=1;I<=Y;I++)
  {
     for(J=1;J<=X;J++)
     {
        MOM[(I-1)*X+J] = MOM[(I-1)*X+J] + DR[(I-1)*X+J];

     }//J
  }//I



  return;


}//add_data


void cal_mean_d(double *DD, int N, double *RR)
{

  int I,J;


  for(I=1;I<=Y;I++)
  {
     for(J=1;J<=X;J++)
     {
        RR[(I-1)*X+J] = DD[(I-1)*X+J] / N;

     }//J
  }//I


  return;


}//cal_mean_d


void cal_var(float *DR)
{

  int I,J;


  for(I=1;I<=Y;I++)
  {
     for(J=1;J<=X;J++)
     {
        STD[(I-1)*X+J] = STD[(I-1)*X+J] + (DR[(I-1)*X+J] - MOM[(I-1)*X+J]) * (DR[(I-1)*X+J] - MOM[(I-1)*X+J]);

     }//J
  }//I


  return;


}//cal_var


void standard_deviation()
{

  int I,J;


  for(I=1;I<=Y;I++)
  {
     for(J=1;J<=X;J++)
     {
        STD[(I-1)*X+J] = sqrt(STD[(I-1)*X+J]);

     }//J
  }//I


  return;


}//standard_deviation


int get_lastday(int Year, int Month)
{

  // Last Day of the Month
  int Last_DD;


  time_t timer;
  struct tm base;
  struct tm *shift;

  base.tm_year = Year-1900;
  base.tm_mon  = Month;
  base.tm_mday = 1;
  base.tm_hour = 0-1;
  base.tm_min  = 0;
  base.tm_sec  = 0;

  timer = mktime(&base);

  shift = localtime(&timer);

  Last_DD = shift->tm_mday;


  return Last_DD;


}//get_lastday


void date_errorcheck()
{

  // Last Day of the Month
  int Last_DD;


  // Year Check
  if(YYYY <= 0)
  {
     printf("Year %d is less than or equal to 0\n", YYYY);
     exit(1);

  }//fi

  // Month Check
  if(MM <= 0 || MM > 12)
  {
     printf("Month %d does not exist\n", MM);
     exit(1);

  }//fi


  return;


}//date_errorcheck


void initialization()
{

  int I,J;


  read_formatfile();


  if((MOM = (double *)malloc(sizeof(double)*((X+1)*(Y+1)))) == NULL) {printf("malloc failed."); exit(1);}
  if((STD = (double *)malloc(sizeof(double)*((X+1)*(Y+1)))) == NULL) {printf("malloc failed."); exit(1);}

  // Array Initialization
  for(I=1;I<=Y;I++)
  {
     for(J=1;J<=X;J++)
     {
        MOM[(I-1)*X+J] = 0;
        STD[(I-1)*X+J] = 0;

     }//J
  }//I


  return;


}//initialization


void read_formatfile()
{

  int I;

  FILE *fp;

  // Present Line
  int pline = 1;

  char filename[512];
  char readline[1000];
  char *param[100+1];


  sprintf(filename, "config/format.txt");


  for(I=0;I<=100;I++)
  {
     if((param[I] = (char *)malloc(sizeof(char)*1000)) == NULL) {printf("malloc failed.\n"); exit(1);}

  }

  printf("%s  Loading ...", filename);

  fp = fopen(filename, "r");

  if(fp == NULL)
  {

     printf("%s read error\n", filename);
     exit(1);

  }//fi

  // Read Format File
  while( fgets(readline, 1000, fp) != NULL)
  {
     strcpy(param[pline], strtok(readline, ", "));

     pline++;

  }//while

  printf(" finish\n");

  fclose(fp);

  sprintf(TIME_Code, "%s", param[2]);

  X = atoi(param[3]);
  Y = atoi(param[4]);

  sprintf(ftype, "%s", param[5]);
  sprintf(DayMn_DIR, "%s", param[7]);

  sprintf(MonthlyMn_DIR, "%s", param[11]);
 
  for(I=0;I<=100;I++) free(param[I]);


  return;


}//read_formatfile

